/************************************************************************
ʰץԡ
************************************************************************/
#ident "$Id: strelay.c 216 2004-12-15 18:48:36Z mkoya $"

#define ENABLE_RCSID

#include "system.h"

#define STDIN 0
#define STDOUT 1

/************************************************************************
argp
************************************************************************/
const char *argp_program_version
= PACKAGE_STRING "\nWritten by " AUTHOR ".";
const char *argp_program_bug_address = AUTHOR " <" PACKAGE_BUGREPORT ">";
static char doc[] = "Simple TCP Relay.";
static char args_doc[] = "hostname port";
static struct argp_option options[] = {
    {"verbose",     'v',   0, 0, "Produce verbose output", 0, },
    {"dumpversion", 0x100, 0, 0, "Print the version only", -1, },
    { 0 }
};

struct arguments
{
    pid_t pid;
    int verbose;
    char *hostname;
    char *port;
};

static error_t parse_opt (int key, char *arg, struct argp_state *state)
{
    struct arguments *args = state->input;
     
    switch (key){
    case ARGP_KEY_INIT:
#if !HAVE_DECL_PROGRAM_INVOCATION_NAME
	set_program_invocation_name(state->argv[0]);
#endif
	args->pid = getpid();
	args->verbose = 0;
	args->hostname = NULL;
	args->port = NULL;
	break;
    case 'v':
	args->verbose = 1;
	break;
    case 0x100:
	puts(PACKAGE_VERSION);
	exit(EXIT_SUCCESS);
    case ARGP_KEY_NO_ARGS:
        argp_usage(state);
    case ARGP_KEY_ARG:
	switch (state->arg_num){
	case 0: args->hostname = arg; break;
	case 1: args->port = arg; break;
	default: state->next = state->argc;
	}
	break;
    case ARGP_KEY_END:
	if (args->port==NULL) argp_usage(state);
	if (args->verbose)
	    error(NOEXIT, NOERRNO, "[%d] Version %s",
		  args->pid, PACKAGE_VERSION);
    default:
	return ARGP_ERR_UNKNOWN;
    }
    return 0;
}

static struct argp argp = {options, parse_opt, args_doc, doc};

/************************************************************************
ݡֹ(ͥåȥХȥ)
************************************************************************/
static uint16_t getportno(const char *port)
{
    struct servent *ent;

    /* ǻϤޤ륵ӥ̾Ϥʤ 80abcdǤ80֤ȸʤ */
    if (isdigit(port[0])) return htons(atoi(port));

    ent = getservbyname(port, "tcp");
    if (ent==NULL) error(EXIT_FAILURE, NOERRNO, "can't get service: %s", port);

    return ent->s_port;
}

/************************************************************************
³
************************************************************************/
static int connectdist(const struct arguments *args)
{
    int dfd;
    struct sockaddr_in addr;
    struct hostent *hp;

    if ((dfd=socket(AF_INET, SOCK_STREAM, 0))<0)
	error(EXIT_FAILURE, errno, "[%d] can't make socket", args->pid);

    memset(&addr, 0, sizeof(addr));

    if ((hp=gethostbyname(args->hostname))==NULL){
#if defined __GLIBC__
	error(EXIT_FAILURE, errno, "[%d] %s", args->pid, args->hostname);
#else
	if (errno)
	    error(EXIT_FAILURE, errno, "[%d] %s", args->pid, args->hostname);
	else
	    error(EXIT_FAILURE, 0, "[%d] %s: %s",
		  args->pid, args->hostname, hstrerror(h_errno));
#endif
    }

    addr.sin_family = AF_INET;
    memcpy(&addr.sin_addr, hp->h_addr, hp->h_length);
    addr.sin_port = getportno(args->port);

    if (connect(dfd, &addr, sizeof(addr))<0)
	error(EXIT_FAILURE, errno, "[%d] can't connect", args->pid);

    if (args->verbose) error(NOEXIT, NOERRNO, "[%d] connected: %s:%s",
		       args->pid, args->hostname, args->port);
    return dfd;
}

/************************************************************************
鱦ر麸
************************************************************************/
static int relay(int dfd, const struct arguments *args)
{
    char buf[BUFSIZ];
    fd_set rfds, ac_rfds;
    int retval;
    int n;
    int recv_count = 0;
    int send_count = 0;

    FD_ZERO(&ac_rfds);
    FD_SET(STDIN, &ac_rfds);
    FD_SET(dfd, &ac_rfds);

    for (;;){
	rfds = ac_rfds;
	retval = select(FD_SETSIZE, SELECT_TYPE_ARG234 &rfds, NULL, NULL,
			NULL);

	if (retval<0) error(EXIT_FAILURE, errno, "[%d] select", args->pid);
	if (retval){
	    if (FD_ISSET(STDIN, &rfds)){
		n = read(STDIN, buf, sizeof(buf));
		if (n<0) error(EXIT_FAILURE, errno, "[%d] read source",
			       args->pid);
		if (n==0){
		    if (args->verbose)
			error(NOEXIT, NOERRNO, "[%d] read eof of soruce",
			      args->pid);
		    FD_CLR(STDIN, &ac_rfds);
		    shutdown(dfd, SHUT_WR);
		}
		else if ((n=write(dfd, buf, n))<0)
		    error(EXIT_FAILURE, errno, "[%d] write source", args->pid);
		else
		    send_count += n;
	    }
	    if (FD_ISSET(dfd, &rfds)){
		n = read(dfd, buf, sizeof(buf));
		if (n<0) error(EXIT_FAILURE, errno, "[%d] read dist",
			       args->pid);
		if (n==0){
		    if (args->verbose)
			error(NOEXIT, NOERRNO, "[%d] read eof of dist",
			      args->pid);
		    break;
		}
		else if ((n=write(STDOUT, buf, n))<0)
		    error(EXIT_FAILURE, errno, "[%d] write dist", args->pid);
		else
		    recv_count += n;
	    }
	}
    }
    close(dfd);
    if (args->verbose)
	error(NOEXIT, NOERRNO, "[%d] disconnected: send %d, recv %d",
	      args->pid, send_count, recv_count);

    return EXIT_SUCCESS;
}

/************************************************************************
main
************************************************************************/
int main(int argc, char **argv)
{
    int dfd;
    struct arguments args;

    argp_parse(&argp, argc, argv, 0, 0, &args);

    dfd = connectdist(&args);

    return relay(dfd, &args);
}
